/*
 * _NET_WM_ICON helpers for slide puzzle game
 * Copyright © 2022 Nick Bowler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef EWMHICON_H_
#define EWMHICON_H_

/*
 * Define X_DISPLAY_MISSING to allow building without X11 (for test purposes)
 */
#if !X_DISPLAY_MISSING
#  include <X11/Intrinsic.h>
#else

/* Stub types to allow compilation */
typedef struct {
	unsigned short red, green, blue;
} XColor;

typedef struct stub_widget *Widget;

#endif

enum { COLOUR_PRIMARY, COLOUR_DARK, COLOUR_LIGHT, COLOUR_MAX };

/*
 * Helpers for drawing window icons in various sizes.  The tileXX functions
 * draw one (out of 9) tiles at a particular size using a particular colour
 * set.  Call for each tile position to draw a complete icon.
 */
enum { ICON_16x16, ICON_24x24, ICON_32x32, ICON_48x48, ICON_MAX };
#define EWMH_ICON_NELEM (2+16*16 + 2+24*24 + 2+32*32 + 2+48*48)

void ewmh_tile16(unsigned long *icon, const XColor *c, int tile_x, int tile_y);
void ewmh_tile24(unsigned long *icon, const XColor *c, int tile_x, int tile_y);
void ewmh_tile32(unsigned long *icon, const XColor *c, int tile_x, int tile_y);
void ewmh_tile48(unsigned long *icon, const XColor *c, int tile_x, int tile_y);

/*
 * Allocate storage for the EWMH _NET_WM_ICON array.  The sizes array is
 * populated with pointers to the beginning of each icon's pixel data.  For
 * example, sizes[ICON_24x24] points to the first pixel of the 24x24 image.
 *
 * The returned value can then be passed to XChangeProperty to set the icon,
 * (use EWMH_ICON_NELEM for the number of elements) and must be freed by the
 * caller.
 */
void *ewmh_icon_alloc(unsigned long **sizes);

/*
 * Check if the root window indicates support for EWMH icons.  Returns 1 if
 * supported, or 0 otherwise.
 */
int ewmh_probe_wm_icon(Widget shell);

#endif
